"""
 Created by db242421 at 14/01/19

 """
from sklearn.model_selection import LeaveOneOut
from sklearn.preprocessing import StandardScaler
from sklearn.svm import SVR
import numpy as np
from scipy.stats import pearsonr
import scipy.special as special
from sklearn.linear_model import Ridge


def _betai(a, b, x):
    x = np.asarray(x)
    x = np.where(x < 1.0, x, 1.0)  # if x > 1 then return 1.0
    return special.betainc(a, b, x)


def vcorrcoef(X, y):
    Xm = np.reshape(np.mean(X, axis=1), (X.shape[0], 1))
    ym = np.mean(y)
    r_num = np.sum((X - Xm) * (y - ym), axis=1)
    r_den = np.sqrt(np.sum((X - Xm) ** 2, axis=1) * np.sum((y - ym) ** 2))
    r = r_num / r_den
    df = len(y) - 2

    t_squared = r ** 2 * (df / ((1.0 - r) * (1.0 + r)))
    prob = _betai(0.5 * df, 0.5, df / (df + t_squared))

    return r, prob


def predict_scores(connectivity_matrices, raw_score,
                   alpha=0.01,
                   C=1,
                   alpha_ridge=1,
                   estimator='svr',
                   with_mean=False,
                   with_std=False):

    sc = StandardScaler(with_std=with_std, with_mean=with_mean)
    # Store the score prediction for each subject
    score_pred = []
    # Store the prediction weight
    score_pred_weights = []
    # Store the selected features indices
    score_selected_features = []
    scores = sc.fit_transform(np.array(raw_score).reshape(-1, 1))
    # initialize a leave one subject out cross validation scheme
    loo = LeaveOneOut()
    for train, test in loo.split(np.arange(0, connectivity_matrices.shape[0], 1)):
        # Split data in train set, and test set
        train_matrices = connectivity_matrices[train]
        train_scores = scores[train, :]

        test_matrices = connectivity_matrices[test]

        # Feature selection with a correlation between score and
        # connectivity for each brain connections
        r_mat, p_mat = vcorrcoef(X=train_matrices.T,
                                 y=np.squeeze(train_scores))

        # Search brain connection surviving the features selection step
        significant_features = np.where(p_mat < alpha)[0]

        # Select the selected brain connection in the train and test set
        test_selected_features = test_matrices[..., significant_features]
        train_selected_features = train_matrices[..., significant_features]

        if estimator == 'svr':

            # Initialize the estimator with the best regularization parameters
            svr = SVR(kernel='linear', C=C)
            # Fit the model on the whole training set
            svr.fit(X=train_selected_features, y=train_scores.ravel())
            # Predict the score of the left out subjects
            score_pred.append(svr.predict(X=test_selected_features)[0])
            # Compute the weight of features of the estimator
            score_pred_weights.append(svr.coef_)
            # Append the indices of selected features
            score_selected_features.append(significant_features)
        elif estimator == 'ridge':
            # Initialize the estimator with the best regularization parameters
            ridge = Ridge(alpha=alpha_ridge)
            # Fit the model on the whole training set
            ridge.fit(X=train_selected_features, y=train_scores.ravel())
            # Predict the score of the left out subjects
            score_pred.append(ridge.predict(X=test_selected_features))
            # Compute the weight of features of the estimator
            score_pred_weights.append(ridge.coef_)
            # Append the indices of selected features
            score_selected_features.append(significant_features)

    # Compute R squared, as the squared correlation coefficient between predicted values, and
    # true scores values.
    r2 = pearsonr(np.array(score_pred), scores[:, 0])[0] ** 2

    return r2, score_pred, score_pred_weights, score_selected_features



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Oct  6 17:04:32 2017

@author: db242421 (dhaif.bekha@cea.fr)


Useful pre-processing step before doing statistical test.
"""

from nilearn.connectome import sym_matrix_to_vec, vec_to_sym_matrix
import numpy as np


def fisher_transform(symmetric_array):
    """Return the Fisher transform of all coefficient in a symmetric array.
    
    Parameters
    ----------
    symmetric_array : numpy.ndarray, shape(..., number of regions, numbers of regions)
        A symmetric numpy array, typically correlation, or partial correlation
        subject connectivity matrices.
    
    Returns
    -------
    output : numpy.ndarray, shape(..., number of regions, numbers of regions)
        The fisher transform of the ndimensional array.
        
    Notes
    -----
    The fisher transform is classically computed according to the 
    formula :
        
        .. math::
            z = arctanh(r)
    
    For **r = 1**, the Fisher transform is not defined, typically on the diagonal
    of connectivity matrices. Then we fill diagonal value with np.nan values.
    
    
    """

    # Discarding diagonal which is one for correlation, and partial correlation, not defined in Z-Fisher transformation
    vectorize_array = sym_matrix_to_vec(symmetric = symmetric_array, discard_diagonal=False)
    
    # Apply Fisher transform, i.e the bijection of hyperbolic tangent:
    fisher_vectorize_array = np.arctanh(vectorize_array)

    # Reconstruct the symmetric matrix:
    fisher_transform_array = vec_to_sym_matrix(vec = fisher_vectorize_array, diagonal=None)
    
    # Number of dimension of fisher transform array
    fisher_array_ndim = fisher_transform_array.ndim
    
    # if ndim = 2, put NaN in the diagonal:
    if fisher_array_ndim == 2:
        
        np.fill_diagonal(a = fisher_transform_array, val = np.nan)
    # if ndim > 2, we consider fisher_transform_array as ndimensional array with shape (..., n_features, n_features),
    # we put nan in the diagonal of each array, along the first dimension
    elif fisher_array_ndim > 2:
        for i in range(fisher_transform_array.shape[0]):
            np.fill_diagonal(a = fisher_transform_array[i,:,:], val = np.nan)

    return fisher_transform_array


def stacked_connectivity_matrices(subjects_connectivity_matrices, kinds):
    """Stacked the connectivity matrices for each group and for each kinds.
    
    If a masked array exist, i.e a 'masked_array' key in each subjects dictionnary, we
    stacked them too.
    
    Parameters
    ----------
    subjects_connectivity_matrices : dict
        A multi-levels dictionnary organised as follow :
            - The first keys levels is the different groupes in the study. 
            - The second keys levels is the subjects IDs
            - The third levels is the different kind matrices
            for each subjects, a 'discarded_rois' key for the
            discarded rois array index, a 'masked_array' key containing
            the array of Boolean of True for the discarded_rois index, and False
            elsewhere.
            
    kinds : list
        List of kinds you are interrested in.
        
    Returns
    -------
    ouput : dict
        A dictionnary with two keys : kinds and masked_array. The values
        is respectively the stacked matrices for one kind in a ndimensional
        array of shape (..., numbers of regions, numbers of regions), and the
        ndimensional boolean mask array of the same shape (..., numbers of regions,
        numbers of regions).
    
    """
    #Dictionnary which will contain the stack of connectivity matrices for each groups and kinds.
    stacked_connectivity_dictionnary = dict.fromkeys(subjects_connectivity_matrices)
    for groupe in subjects_connectivity_matrices.keys():
        #Dictionnary initialisation with all kinds as entry
        stacked_connectivity_dictionnary[groupe] = dict.fromkeys(kinds)
        #subjects list for the groupe
        subjects_list = subjects_connectivity_matrices[groupe].keys()
        for kind in kinds:
            #We stack the matrices for the current kind
            kind_groupe_stacked_matrices = [subjects_connectivity_matrices[groupe][subject][kind] for subject in subjects_list]
            #We detect if the keys named 'masked_array' existe to create a numpy masked array strucutre.
            kind_masked_array = [ subjects_connectivity_matrices[groupe][subject]['masked_array'] for subject in subjects_list if 'masked_array' in subjects_connectivity_matrices[groupe][subject].keys() ]
            
            #We fill a dictionnary with kind connectivity as keys, and stacked connectivity matrices, with masked array if they exist as values
            stacked_connectivity_dictionnary[groupe][kind] = np.array(kind_groupe_stacked_matrices)
        
            stacked_connectivity_dictionnary[groupe]['masked_array'] = np.array(kind_masked_array)
            
    return stacked_connectivity_dictionnary



# -*- coding: utf-8 -*-
"""
Created on Thu Dec 22 19:07:13 CET 2019

@author:  edouard.duchesnay@cea.fr
@license: BSD-3-Clause
"""

import numpy as np
import pandas as pd
import patsy
from collections import OrderedDict

def design_matrix(formula, data):
    """
    Build the design matrix t-contrasts and F-contrasts given the formula and the dataframe.
    Remark: use `patsy.dmatrix(formula, data=data)`

    Parameters
    ----------

    formula, str
    data, DataFrame

    return
    ------
    X: (array) the design matrix
    t_contrasts: (OrderedDict key=colname, val=array (P,)). Remark: len(t_contrasts) == X.shape[1]
    f_contrasts: (OrderedDict key=term name in formula, val=array (P, P)),
    """

    # Build design matrix
    dmat = patsy.dmatrix(formula, data=data)

    # Build T-contrasts
    t_contrasts = OrderedDict()

    for idx, name in enumerate(dmat.design_info.column_names):
        cont = np.zeros(dmat.shape[1])
        cont[idx] = 1
        t_contrasts[name] = cont

    # Build F-contrasts
    f_contrasts = OrderedDict()

    for term, slice in dmat.design_info.term_name_slices.items():
        # if term == "site":
        #    break
        cont = np.zeros((dmat.shape[1], dmat.shape[1]))
        indices = np.arange(slice.start, slice.stop)
        cont[indices, indices] = 1
        f_contrasts[term] = cont

    return np.asarray(dmat), t_contrasts, f_contrasts



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 18 17:32:38 2017

@author: Dhaif BEKHA

ComPagnon version 2.0

"""
import os
import pandas as pd
import re
import glob

from conpagnon.utils import folders_and_files_management as ffm


def read_text_data_file(file_path, colname=None, header=None):
    """Read a data file 
    
    The data file can be a .csv, .txt, .xlsx or .xls file
    
    Parameters
    ----------
    
    file_path : str
        Full path to the file to read.
    colname : None or str
        The column name to extract.
    header : None of int
        Row number to use as the column names, and the start of the data.
        
    Returns
    -------
    
    output : pandas.core.frame.DataFrame
        The extracted column if form of a panda dataframe.
    
    """
    
    if file_path.lower().endswith(('.csv', '.txt')):
        data = pd.read_csv(file_path, header = header)
    elif file_path.lower().endswith(('.xlsx', '.xls')):
        data = pd.read_excel(file_path, header = header)
        
    if colname:
        data = data[colname]
    
    return data


def list_fmri_data(root_fmri_data_directory):
    """Fetch all functional images found in sub-directories
    at a root directory.
    
    Parameters
    ----------
    root_fmri_data_directory : str
        The full path of a root directory containing one or numerous
        sub-directories where functional images are.
    
    
    Returns
    -------
    output : dict
        A dictionnary with sub-directories names as keys and full path
        to functional images as values.
    """
    # Initialisation of list which will contain full path to functional images
    fmri_files = []
    
    # Check if sub-directories are empty, delete them if this is the case
    ffm.remove_empty_directories(root_directory = root_fmri_data_directory)
    
    for path, dirs, files in os.walk(root_fmri_data_directory):
        # List of the different sub-directories
        if dirs != []:
            groupes = dirs
        # Append fmri files
        if files != []:
            fmri_files.append(files)
                
    # Creating the dictionnary with sub-directories names as keys, i.e
    # the different groups of the study and full path to functional images
    # as values
    n_groupes = len(groupes)
    fmri_files_dict = dict([(groupes[groupe], fmri_files[fmri_file])
                            for groupe, fmri_file in zip(range(n_groupes),range(n_groupes)) ])

    # Filling the dictionnary values with full path to fmri files for each keys
    for groupe in fmri_files_dict.keys():
        # Name of the images
        list_name_img = fmri_files_dict[groupe]
        # Appending the full path
        list_img_fullpath = [ os.path.join(root_fmri_data_directory, groupe) + '/' + name_img 
                             for name_img in list_name_img]
        fmri_files_dict[groupe] = list_img_fullpath
        del list_name_img
        
    return fmri_files_dict


def fetch_fmri_data(root_fmri_data_directory, groupes):
    """Fetch functional images found in a list of 
    sub-directories.
    
    Parameters
    ----------
    
    root_fmri_data_directory : str
        The full path of a root directory containing one or numerous
        sub-directories where functional images are.
        
    groupes : list
        List of sub-directories names containing fmri files you want.
    
    
    Returns
    -------
    
    output : dict
        A dictionnary with sub-directories names as keys and full path
        to functional images as values.
        
    See Also
    --------
    
    list_fmri_data: Fetch functional images in all sub-directories.
    
    """
    # We fetch all fmri data thank to list_fmri_data
    fmri_files_dict = list_fmri_data(root_fmri_data_directory=root_fmri_data_directory)
    
    # We extract a sub-dictionnary according to the list entered in groupes:
    # We check that the elements of groupes are existing keys in fmri_files_dict
    for i in range(len(groupes)):
        if groupes[i] not in fmri_files_dict.keys():
            raise ValueError('Group {} doesn\'t exist !. available groups in {} are \n {}'.format(groupes[i],
                             root_fmri_data_directory, fmri_files_dict.keys()))
        else:
            pass
            
    # If keys exist, we can extract the sub-dictionnary to return the new ones
    group_fmri_dict = dict((g, fmri_files_dict[g]) for g in groupes)
    
    return group_fmri_dict


def create_group_dictionnary(subjects_id_data_path, root_fmri_data_directory, groupes):
    """Initialise a dictionnary containing groups as keys and subjects IDs as values
    
    Parameters
    ----------
    
    subjects_id_data_path : str
        The full path to the data file containing the subjects IDs.
            
    root_fmri_data_directory : str
        The full path of a root directory containing one or numerous
        sub-directories where functional images are
        
    groupes : list
        List of sub-directories names containing fmri files you want.
    
    
    Returns
    -------
    
    output : dict
        A dictionnary groupes as keys and subjects IDs as values for each
        groups
        
    See Also
    --------
    
    list_fmri_data : Fetch functional images in all sub-directories.
    read_text_data_file : read a text data file.
    
    Notes
    -----
    
    Whatever the format for the subjects IDs datafile, it should not
    contains any header. it should consist of one raw columns of subject 
    IDs.
    
    
    """
    dict2 = dict.fromkeys(groupes)
    subjects_ID = read_text_data_file(file_path=subjects_id_data_path, colname=None, header=None)
    for groupe in groupes:
        # List available functional images
        a = glob.glob(os.path.join(root_fmri_data_directory, groupe, '*'))
        # Search and extraction of subjects identifier
        b= [re.search(s_id,f_name) for f_name in a for s_id in subjects_ID[0]]
        #  Fetch the subjects identifier present
        c = [b[i].group(0) for i in range(len(b)) if b[i] != None]
        # Create a subject identifier dictionary per group
        dict_sub_id = dict.fromkeys(c)

        dict2[groupe] = dict_sub_id
        
    return dict2


def fetch_data_with_individual_atlases(subjects_id_data_path, root_fmri_data_directory, groupes,
                                       individual_atlases_directory, individual_atlases_labels_directory,
                                       individual_atlas_file_extension, individual_atlas_labels_extension,
                                       individual_counfounds_directory=None):
    """Fetch a complete organised structure for a groups study require the use
    of individual atlases
    
    
    Parameters
    ----------
    subjects_id_data_path: str
        The full path to the data file containing the subjects IDs.
        
    root_fmri_data_directory: str
        The full path of a root directory containing one or numerous
        sub-directories where functional images are
        
    groupes: list
        List of sub-directories names containing fmri files you want.
        
    individual_atlases_directory: str
        Full path to individual atlases directory for all subjects.
        
    individual_atlases_labels_directory: str
        Full path to individual atlases labels directory for all subjects.
        
    individual_atlas_file_extension: str
        Extension of individuals atlases images for all subjects.
        
    individual_atlas_labels_extension: str
        Extension of text data containing individual atlases labels file for
        all subjects.
        
    individual_counfounds_directory : None or str
        Full path to counfounds files for all subjects.

    Returns
    -------
    output: dict
        A multi level dictionnary containing all the data. 
        The first level is the groups keys. The second levels is the subjects 
        IDs. The last level, is all the relevant file for one subjects: fmri image,
        subject atlas, subject atlas labels file, and confound file if required.
        A keys called 'discarded_rois' contain the excluded rois, see Notes.
    
    See Also  
    --------
    fetch_data : Fetch a complete organised structure for a groups study
    on a common atlas for all subjects.
    
    Notes
    -----
    Whatever the format for the subjects IDs datafile, it should not
    contains any header. it should consist of one raw columns of subject 
    IDs.
    
    A discarded_rois is a subject atlas ROI where the corresponding
    labels is 'void'. This ROIS can be a empty ROIs, a ROI you doesnt need
    for the analysis. The discarded rois will be discarded (!) in the 
    connectivity analysis when computing t-test for example. 
    
    
    """
    
    group_fmri_dict = fetch_fmri_data(root_fmri_data_directory=root_fmri_data_directory,
                                      groupes=groupes)

    subjects_ID = read_text_data_file(file_path=subjects_id_data_path, colname=None,
                                      header=None)
    
    # Full path to individual atlases files and labels file directory
    individual_atlases_files = glob.glob(os.path.join(individual_atlases_directory,
                                                      individual_atlas_file_extension))
    individual_atlases_labels_files = glob.glob(os.path.join(individual_atlases_labels_directory,
                                                             individual_atlas_labels_extension))
    
    # Check if counfounds files is required
    if individual_counfounds_directory != None:
        individual_confounds_files = glob.glob(os.path.join(individual_counfounds_directory, '*'))
    else:
        individual_confounds_files = False

    dict1 = {}
    n_groupes = len(groupes)

    for s_id in subjects_ID[0]:
        for groupe in range(n_groupes):
            # Subjects ID to look for
            regex = re.compile(".*("+str(s_id)+").*")
            # Fetch the correspond fmri file
            subject_functional_file = [m.group(0) for l in group_fmri_dict[groupes[groupe]]
                                       for m in [regex.search(l)] if m]
            # If fmri exist, fetch the corresponding atlas file, and atlas labels files
            if subject_functional_file != []:
                # Subject atlas
                subject_atlas_file = [m.group(0) for l in individual_atlases_files
                                      for m in [regex.search(l)] if m]
                # Subject atlas labels
                subject_atlas_labels_file = [m.group(0) for l in individual_atlases_labels_files
                                             for m in [regex.search(l)] if m]
                
                # Append confound if they exist:
                if individual_confounds_files:
                    subject_counfounds_file = [m.group(0) for l in individual_confounds_files
                                               for m in [regex.search(l)] if m]
                    
                else:
                    subject_counfounds_file = []
                
                # Creation of a sub-dictionary for each subject containing the full
                # path to files as values, and file descriptions as keys
                subject_files = {'functional_file': subject_functional_file,
                                 'atlas_file': subject_atlas_file,
                                 'label_file': subject_atlas_labels_file,
                                 'counfounds_file': subject_counfounds_file}
                # Appending the sub-dictionnary to the corresponding subjects entries
                dict1[s_id] = subject_files
    
    # We initialize the final dictionnary with groups as keys and subject ID as values
    dict2 = create_group_dictionnary(subjects_id_data_path=subjects_id_data_path,
                                     root_fmri_data_directory=root_fmri_data_directory,
                                     groupes=groupes)

    # We fill the final dictionnary with the information gathered for each subject
    for groupe in groupes:
        for s_id in subjects_ID[0]:
            if s_id in dict2[groupe].keys():
                 dict2[groupe][s_id] = dict1[s_id]

    return dict2


def fetch_data(subjects_id_data_path, root_fmri_data_directory, groupes,
               individual_confounds_directory=None):
    """Fetch a complete organised structure for a groups study on a common
    atlas for all subjects
    
    
    Parameters
    ----------
    subjects_id_data_path : str
        The full path to the data file containing the subjects IDs.
        
    root_fmri_data_directory : str
        The full path of a root directory containing one or numerous
        sub-directories where functional images are
        
    groupes : list
        List of sub-directories names containing fmri files you want.
        
    individual_confounds_directory : None or str
        Full path to confounds files for all subjects.


    Returns
    -------
    output : dict
        A multi level dictionnary containing all the data. 
        The first level is the groups keys. The second levels is the subjects 
        IDs. The last level, is all the relevant file for one subjects: fmri image,
        and confound file if required.
    
    See Also  
    --------
    fetch_data_with_individual_atlases : Fetch a complete organised
    structure for a groups study which require an individual atlas per subjects.
    
    Notes
    -----
    Whatever the format for the subjects IDs datafile, it should not
    contains any header. it should consist of one raw columns of subject 
    IDs.

    """
    
    group_fmri_dict = fetch_fmri_data(root_fmri_data_directory=root_fmri_data_directory,
                                      groupes=groupes)

    subjects_ID = read_text_data_file(file_path=subjects_id_data_path,
                                      colname=None, header=None)
    
    if individual_confounds_directory != None:
        individual_confounds_files = glob.glob(os.path.join(individual_confounds_directory, '*'))
    else:
        individual_confounds_files = False

    dict1 = {}

    n_groupes = len(groupes)

    for s_id in subjects_ID[0]:
        for groupe in range(n_groupes):
            # the subjects ID to look for
            regex = re.compile(".*("+str(s_id)+").*")
            # Fetch of fmri image
            subject_functional_file = [m.group(0) for l in group_fmri_dict[groupes[groupe]]
                                       for m in [regex.search(l)] if m]
            # If fmri exist we look for a confound file
            if subject_functional_file != []:                
                # If a confound file we append it:
                if individual_confounds_files:
                    subject_counfounds_file = [m.group(0) for l in individual_confounds_files
                                               for m in [regex.search(l)] if m]
                else:
                    subject_counfounds_file = []
                
                # Creation of a sub-dictionary for each subject containing the full path
                # to files as values, and file descriptions as keys
                subject_files = {'functional_file': subject_functional_file,
                                 'counfounds_file': subject_counfounds_file}
                # We append the sub-dictionnary to the subject dictionnary for each subject
                dict1[s_id] = subject_files
    
    # We initialize the final dictionnary with groups as keys and subject ID as values
    dict2 = create_group_dictionnary(subjects_id_data_path=subjects_id_data_path,
                                     root_fmri_data_directory=root_fmri_data_directory,
                                     groupes=groupes)

    # We fill the final dictionnary with the information gathered for each subject
    for groupe in groupes:
        for s_id in subjects_ID[0]:
            if s_id in dict2[groupe].keys():
                 dict2[groupe][s_id] = dict1[s_id]

    return dict2





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 28 09:58:24 2017

@author: Dhaif BEKHA


Useful operation on arrays.
"""

import numpy as np
from numpy import ma
from nilearn.connectome import sym_matrix_to_vec, vec_to_sym_matrix
import re
from nilearn.image import concat_imgs
import nibabel as nb
import glob
import os
from math import sqrt


def masked_void_rois_connectivity_matrix(subject_kinds_connectivity, kinds,
                                         discard_diagonal=False,
                                         vectorize=False):

    """Compute a boolean mask array for discarded rois 
    
    Parameters
    ----------
    subject_kinds_connectivity: dict
        The individual subject dictionnary containing as keys:
            - The different kinds
            - A keys called 'discarded_rois' for ROIs to discard during analysis
            
    kinds : list 
        List of present kinds in the subjects connectivity matrices
        dictionnary.

    discard_diagonal: bool, optional
        If True, the diagonal are discarded in the vectorization
        process. Default is False, the diagonal is kept.

    vectorize: bool, optional
        If True, the returned mask is vectorized. Default
        is False

    Returns
    -------
    output : numpy.array, shape (number of regions, number of regions)
        A boolean array : True when rois are considered discarded, False elsewhere
        See notes concerning the discarded roi
        
    See Also
    --------
    compute_connectivity_matrices.individual_connectivity_matrices :
        This is the function which compute the connectivity matrices for
        the chosen kinds, and return a structured dictionnary containing
        all the connectivity matrices for each subjects and kinds
        
    Notes
    -----
    A discarded rois, is a ROI where the corresponding labels is 'void' in the
    individual atlas of a given subject. This information is automatically gathered 
    when you extract time series with an individual atlas for each subjects. We account
    for discarded rois when computing arithmetic mean, or performing t-test for
    examples by excluding them during the computation.
    
    Excluding the discarded rois, is possible via the numpy.ma structure which
    build a numpy masked array that is a couple of a numerical array and a boolean
    array where True are masked values, and False are un-masked values.
    
    Please refer to the numpy documentation for further details.
    """
    # We fetch the number of regions
    # Get the shape of the matrices
    matrix_shape = subject_kinds_connectivity[kinds[0]].shape

    # We fetch the index of discarded rois
    subject_empty_roi = subject_kinds_connectivity['discarded_rois']
    
    # Masked array: True for discarded rois, else elsewhere.

    # If we have a 2D subjects connectivity matrices, we initialize a 2D boolean mask
    if not vectorize:
        numbers_of_regions = matrix_shape[0]
        mask = np.invert(ma.make_mask(np.ones((numbers_of_regions, numbers_of_regions))))
        if subject_empty_roi.size:
            for empty_roi in subject_empty_roi:
                # True for the entire rows corresponding to the current empty roi
                mask[:, empty_roi] = True
                # True for the entire columns corresponding to the current empty roi
                mask[empty_roi, :] = True
        # Compute the diagonal of the mask
        diag_mask = np.diagonal(mask)
    else:
        if discard_diagonal:
            c = len(subject_kinds_connectivity[kinds[0]])
            numbers_of_regions = int(((sqrt(8 * c + 1) - 1.) / 2) + 1)
            # Initialise a vectorized mask
            mask = np.invert(ma.make_mask(np.ones(numbers_of_regions)))
            # Reconstruct the mask to a matrix
            _ , mask_m = vectorizer(numpy_array=mask, discard_diagonal=discard_diagonal,
                                  array_type='boolean')
            # Fill the corresponding empty ROI with True value
            if subject_empty_roi.size:
                for empty_roi in subject_empty_roi:
                    # True for the entire rows corresponding to the current empty roi
                    mask_m[:, empty_roi] = True
                    # True for the entire columns corresponding to the current empty roi
                    mask_m[empty_roi, :] = True

            # Re-vectorized the matrix boolean mask discarding
            # the diagonal
            diag_mask, mask = vectorizer(numpy_array=mask_m,
                                         discard_diagonal=discard_diagonal,
                                         array_type='boolean')
        else:
            c = len(subject_kinds_connectivity[kinds[0]])
            numbers_of_regions = int(((sqrt(8 * c + 1) - 1.) / 2))
            # Initialise a vectorized mask
            mask = np.invert(ma.make_mask(np.ones(numbers_of_regions)))
            # Reconstruct the mask to a matrix
            _, mask_m = vectorizer(numpy_array=mask, discard_diagonal=False,
                                   array_type='boolean')
            # Fill the corresponding empty ROI with True value
            if subject_empty_roi.size:
                for empty_roi in subject_empty_roi:
                    # True for the entire rows corresponding to the current empty roi
                    mask_m[:, empty_roi] = True
                    # True for the entire columns corresponding to the current empty roi
                    mask_m[empty_roi, :] = True
            # If discard_diagonal is False, we vectorized
            # the mask keeping the diagonal
            diag_mask, mask = vectorizer(numpy_array=mask_m,
                                         discard_diagonal=False,
                                         array_type='boolean')

    return mask, diag_mask


def append_masks(subjects_connectivity_dictionnary, kinds,
                 discard_diagonal, vectorize):

    """Append a new key called 'masked_array' in the subjects connectivity
    matrices dictionnary and stock as values a boolean mask array accounting
    for discarded rois.
    
    Parameters
    ----------
    subjects_connectivity_dictionnary : dict
        A multi-levels dictionnary organised as follow :
            - The first keys levels is the different groupes in the study. 
            - The second keys levels is the subjects IDs
            - The third levels is the different kind matrices
            for each subjects, a 'discarded_rois' key for the
            discarded rois array index.
    
    kinds : list
        List of present kinds in the subjects connectivity matrices
        dictionnary.

    discard_diagonal: bool
        If True, the mask diagonal is discarded in the
        vectorization process.
        This argument is passed to masked_void_rois_connectivity_matrix

    vectorize: bool
        If True, the returned mask is vectorized.
        This argument is passed to masked_void_rois_connectivity_matrix

    Returns
    -------
    output : dict
        The modified subjects connectivity matrices with a new key for
        each subjects containing the boolean mask array for the discarded ROIs.
        A new keys is added too, which contain the diagonal array of the mask.

        
    Notes
    -----
    If the discarded rois keys do not exist, that is no labels 'void' is found 
    in the atlas labels file of each subjects, then all roi are included in the
    analysis, and we append a boolean mask array of False values.
    
    
    """
    for groupe in subjects_connectivity_dictionnary.keys():
        subjects_list = subjects_connectivity_dictionnary[groupe].keys()
        for subject in subjects_list:
            # Entries in subject dictionnary, for the empty rois field.
            subject_kind_connectivity_matrice = subjects_connectivity_dictionnary[groupe][subject]
            # Creation of the mask array for void rois based on the field 'empty_rois'
            if 'discarded_rois' in subjects_connectivity_dictionnary[groupe][subject].keys():
                if vectorize:
                    if discard_diagonal:
                        subject_mask, subject_diag_mask = masked_void_rois_connectivity_matrix(
                            subject_kinds_connectivity=subject_kind_connectivity_matrice,
                            kinds=kinds, discard_diagonal=discard_diagonal,
                            vectorize=vectorize)
                    else:
                        subject_mask, subject_diag_mask = masked_void_rois_connectivity_matrix(
                            subject_kinds_connectivity=subject_kind_connectivity_matrice,
                            kinds=kinds, discard_diagonal=False,
                            vectorize=vectorize)

                    # We create new keys containing the masked array.
                    subjects_connectivity_dictionnary[groupe][subject]['masked_array'] = subject_mask
                    # And the corresponding diagonal
                    subjects_connectivity_dictionnary[groupe][subject]['diagonal_masked_array'] = subject_diag_mask
                else:
                    subject_mask, subject_diag_mask = masked_void_rois_connectivity_matrix(
                        subject_kinds_connectivity=subject_kind_connectivity_matrice,
                        kinds=kinds, discard_diagonal=False,
                        vectorize=False)

                    # We create new keys containing the masked array.
                    subjects_connectivity_dictionnary[groupe][subject]['masked_array'] = subject_mask
                    subjects_connectivity_dictionnary[groupe][subject]['diagonal_masked_array'] = subject_diag_mask
            else:
                subject_mask = np.invert(ma.make_mask(np.ones(
                    subjects_connectivity_dictionnary[groupe][subject][kinds[0]].shape)))
                if vectorize:
                    if discard_diagonal:
                        subject_diag_mask, subject_mask = vectorizer(numpy_array=subject_mask,
                                                                     discard_diagonal=discard_diagonal,
                                                                     array_type='boolean')
                    else:
                        subject_diag_mask, subject_mask = vectorizer(numpy_array=subject_mask,
                                                                     discard_diagonal=False,
                                                                     array_type='boolean')

                    subjects_connectivity_dictionnary[groupe][subject]['masked_array'] = subject_mask
                    subjects_connectivity_dictionnary[groupe][subject]['diagonal_masked_array'] = subject_diag_mask
                else:
                    subject_mask = subject_mask
                    subjects_connectivity_dictionnary[groupe][subject]['masked_array'] = subject_mask

    new_subjects_connectivity_dictionnary = subjects_connectivity_dictionnary
    
    return new_subjects_connectivity_dictionnary


def masked_arrays_mean(arrays, masked_array, axis):
    """Compute the arithmetic mean of a ndimensional numpy masked array.
    
    Parameters
    ----------
    arrays : ndimensional numpy.array, shape (..., number of regions, number of regions)
        The ndimensional array you want to compute the mean.
        
    masked_array : ndimensional numpy.array, shape (..., numbers_of_regions, numbers_of_regions)
        The ndimensional boolean masked accounting for the values you want to discard.
        True for masked values, False elsewhere.
    
    axis : int
        The direction for computing the mean.
    
    Returns
    -------
    output : numpy.array shape (number of regions, number of regions)
        The mean array excluding some value in the direction along axis.
    
    See Also
    --------
    masked_void_rois_connectivity_matrix :
        This function compute the boolean mask for accounting
        the discarded rois.
    
    """
    # Initialization of masked array: a couple of a classical numpy array along with
    # a boolean array of the same shape
    mean_array_masked = np.ma.array(data=arrays, mask=masked_array)
    if masked_array.shape != arrays.shape:
        raise ValueError('Array dimension is {}, but the '
                         'corresponding mask is shape {}'.format(arrays.shape, masked_array.shape))
    
    mean_array = mean_array_masked.mean(axis=axis).data
    
    return mean_array


def mask_array_std(arrays, masked_array, axis):
    """Compute the standard deviation of a ndimensional numpy array.
    
    Parameters
    ----------
    arrays : ndimensional numpy.array, shape (number of subjects, number of regions, number of regions)
        The ndimensional array you want to compute the mean.
        
    masked_array : ndimensional numpy.array, shape (..., numbers_of_regions, numbers_of_regions)
        The ndimensional boolean masked accounting for the values you want to discard.
        True for masked values, False elsewhere.
    
    axis : int
        The direction for computing the standard deviation.
    
    Returns
    -------
    output : numpy.array shape (number of regions, number of regions)
        The standard deviation array excluding some value in the direction along axis.
    
    
    See Also
    --------
    masked_void_rois_connectivity_matrix :
        This function compute the boolean mask for accounting
        the discarded rois.
    
    """
    
    # Masked array initialization with numpy masked methods.
    std_array_masked = np.ma.array(data=arrays, mask=masked_array)
    if masked_array.shape != arrays.shape:
        raise ValueError('The numerical array have shape {}, but '
                         'the corresponding mask is shape {} !'.format(arrays.shape,
                                                                       masked_array.shape))

    std_array = std_array_masked.std(axis=axis)
    
    return std_array


def vectorize_boolean_mask(symmetric_boolean_mask, discard_diagonal=False):
    """Vectorize the lower part of symmetric boolean mask. 
    
    Parameters
    ----------
    symmetric_boolean_mask : numpy.array, shape(..., numbers of regions, numbers of regions)
        A ndimensional boolean numpy array.
        
    discard_diagonal : bool, optional
        If True, the values of the diagonal are not returned.
        Default is False.
        
    Returns
    -------
    output : numpy.ndarray
        The flatten ndimensional boolean mask of shape (..., n_features * (n_features + 1) / 2) 
        if discard_diagonal is False and (..., (n_features - 1) * n_features / 2) otherwise.
        
    See Also
    --------
    nilearn.connectome.sym_matrix_to_vec, vec_to_sym_matrix :
        Useful Nilearn function to vectorize and reconstruct 
        symmetric array like the connectivity matrices.
    
    vec_to_matrix_boolean_mask :
        This function reconstruct a flatten boolean ndimensional array.
        
    """
    # Convert the boolean mask to a numerical array
    numerical_mask = np.asanyarray(symmetric_boolean_mask, dtype=int)

    # vectorize the mask
    symmetric_boolean_mask_vectorized = sym_matrix_to_vec(symmetric=numerical_mask, discard_diagonal=discard_diagonal)
    
    if not discard_diagonal:
        symmetric_boolean_mask_vectorized = np.asanyarray(symmetric_boolean_mask_vectorized, dtype=bool)

    return symmetric_boolean_mask_vectorized


def vec_to_matrix_boolean_mask(vectorized_mask, diagonal=None):
    """Reconstruct a ndim symmetrical boolean mask array giving a boolean vectorized mask.
    
    Parameters
    ----------
    vectorized_mask : numpy.ndarray, shape(..., number of regions, number of regions)
        The vectorized boolean mask.    
    
    
    diagonal : numpy.array of shape(number of regions), optional
        The diagonal to reconstruct if you discard the diagonal when
        you first vectorize the ndimensional mask array
        
    Returns
    -------
    output : numpy.ndarray
        The reconstruct ndimensional array of shape (..., numbers of regions, numbers of regions)
        
    See Also
    --------
    nilearn.connectome.sym_matrix_to_vec, vec_to_sym_matrix :
        Useful Nilearn function to vectorize and reconstruct 
        symmetric array like the connectivity matrices.
    
    vectorize_boolean_mask :
        This function flatten a 
        boolean ndimensional array.
    """
    # mask reconstruction
    numerical_mask = vec_to_sym_matrix(vec=vectorized_mask, diagonal=diagonal)
    
    # Convert numerical mask to the boolean mask:
    symmetric_boolean_mask_reconstructed = np.asanyarray(numerical_mask, dtype=bool)

    return symmetric_boolean_mask_reconstructed


def vectorizer(numpy_array, discard_diagonal=False, array_type='numeric'):
    """Vectorize a numerical or boolean numpy array

    Parameters
    ----------
    numpy_array: numpy.array, shape (n_features, n_features)
        A two-dimensional numerical, or boolean data type array.

    discard_diagonal: bool, optional
        If True, the lower triangle of the array is flatten without the diagonal. Default is False.

    array_type: string, optional
        Precise the array type, choices are {'numeric', boolean'}, default is 'numeric'.

    Returns
    -------
    output 1: numpy.array, shape(n_features)
        The diagonal of the entered array.
    output 2: numpy.array, shape( (n_features*(n_features+1))/2 )
    is discard_diagonal is True, (n_features*(n_features-1))/2 else.
        The vectorized array.

    """

    # Retrieve the array diagonal
    array_diagonal = np.diag(numpy_array)

    # if it's a numerical array
    if array_type == 'numeric':
        vectorized_array = sym_matrix_to_vec(numpy_array, discard_diagonal=discard_diagonal)
    elif array_type == 'boolean':

        # two case: first discard diagonal is False
        if discard_diagonal is False:
            vectorized_array = vectorize_boolean_mask(symmetric_boolean_mask=numpy_array, discard_diagonal=False)
        else:
            vectorized_array = vectorize_boolean_mask(symmetric_boolean_mask=numpy_array, discard_diagonal=True)
            # vectorized_array is numerical, we force it to boolean type
            vectorized_array = np.asanyarray(vectorized_array, dtype=bool)
    else:
        raise ValueError('Unrecognized array type, possible choices are: \n '
                         'numeric or boolean, and you entered:{}'.format(array_type))

    return array_diagonal, vectorized_array


def array_rebuilder(vectorized_array, array_type, diagonal=None):
    """Reconstruct a square array assuming a it is symmetric

    Parameters
    ----------
    vectorized_array: numpy.array
        The one dimensional array you want to rebuild.
    diagonal: numpy.array, optional
        The one dimensional array containing the diagonal of
        the non-vectorized array.
        If None, we assume that the diagonal was kept in the
        vectorization process.
    array_type: str
        The type of the array.
        Choices are: numeric, bool.

    Returns
    -------
    output 1: numpy.array
        The reconstructed array, shape (n_features, n_features).

    See Also
    --------
    vectorizer:
        This function vectorized a two dimensional numeric
        or boolean array
    nilearn.connectome.vec_to_sym_matrix:
        This function reconstruct a symmetric array from a one
        dimensional array.
    """

    if array_type == 'numeric':
        # We divide by sqrt(2), preserving the norm if the user
        # give a diagonal
        if diagonal is not None:
            rebuild_array = vec_to_sym_matrix(vec=vectorized_array,
                                              diagonal=(1/sqrt(2))*diagonal)
        else:
            # If the user doesn't give a diagonal we assume it
            # was vectorized without discarding the diagonal

            # rebuild the array
            rebuild_array = vec_to_sym_matrix(vec=vectorized_array,
                                              diagonal=None)

    elif array_type == 'bool':
        if diagonal is not None:
            # If the user give the boolean diagonal
            # Check if the diagonal is a boolean
            if np.issubdtype(diagonal.dtype, np.bool):

                rebuild_array = np.array(vec_to_sym_matrix(vec=vectorized_array,
                                                           diagonal=diagonal),
                                         dtype='bool')
            else:
                raise TypeError('Diagonal should be type bool, and you give type {}'.
                                format(np.dtype(diagonal)))
        else:
            # If the user doesn't give a diagonal we assume
            # the mask was vectorized with the diagonal

            # We rebuild the boolean mask
            rebuild_array = np.array(vec_to_sym_matrix(vec=vectorized_array,
                                                       diagonal=None), dtype='bool')

    else:
        raise ValueError('Array type not understood, choices are: numeric or bool')

    return rebuild_array


def concatenate_imgs_in_order(imgs_directory, index_roi_number, filename,
                              save_directory):
    """Concatenate 3D NifTi images into a single 4D NifTi files, given a user order.

    Parameters
    ----------
    imgs_directory: string
        The directory containing the list of images. Each file, must contain a number
        identifying the file.
    index_roi_number: list of int
        The list of file number in the order you want to ordered
        the images before concatenation
    filename: string
        The final name of the 4D file containing the images.
    save_directory: string
        The directory where to save the 4D generated 4D file.

    Returns
    -------
    output: list
        Return the list of the pull path to roi files,
        in the order of concatenation. It should exactly follow the order you provide
        via the variable `index_roi_number`.

    See Also:
    --------
    nilearn.image.concat_img: This function concatenate a list of 3D NifTi images.
        We use this function to concatenate the images.

    Notes
    -----
    Each file you want to concatenate, must have a pattern to identifying them. Indeed,
    we use it to search the entire path corresponding to the file, and append it in a list
    containing all the path to the files in the right order before concatenation. The pattern
    is simply a number in the file name.

    """
    unordered_roi_list = glob.glob(os.path.join(imgs_directory, '*.nii'))

    # Fetch the correspond fmri file
    ordered_roi_list = []
    for roi in index_roi_number:
        # Search the string containing the roi_index we want
        regex = re.compile(".*(" 'roi_' + str(roi) + '.nii' ").*")
        roi_path = [m.group(0) for l in unordered_roi_list for m in [regex.search(l)] if m]
        # Append the right path
        ordered_roi_list.append(roi_path[0])

    # Concatenate the image following the order in ordered_roi_list
    img4d = concat_imgs(ordered_roi_list)

    # Save the new image
    nb.save(img=img4d, filename=os.path.join(save_directory, filename))

    return ordered_roi_list


def check_2d(numpy_array):
    """Check if an array is two dimensional.

    Parameters
    ----------
    numpy_array: numpy.array of any shape.

    Raises
    ------
    ValueError : if the passed array is NOT two dimensional, a Value Error
    is raised.

    """

    if numpy_array.ndim != 2:
        raise ValueError('Expected two dimensional array but '
                         'passed array shape is {}'.format(numpy_array.shape))


def truncate(f, n):
    """Truncates/pads a float f to n decimal places without rounding

    """

    s = '{}'.format(f)
    if 'e' in s or 'E' in s:
        return '{0:.{1}f}'.format(f, n)
    i, p, d = s.partition('.')
    return '.'.join([i, (d+'0'*n)[:n]])



# -*- coding: utf-8 -*-
"""
Created on Fri Jun 14 19:38:46 2013

@author: ed203246
"""

from .models import MUPairwiseCorr
from .models import MUOLS
from .helpers import design_matrix

__all__ = ['MUPairwiseCorr',
           'MUOLS',
           'design_matrix']



from setuptools import find_packages, setup


DISTNAME = 'conpagnon'
DESCRIPTION = 'Easy Resting State Analysis with Python'
with open('README.rst') as fp:
    LONG_DESCRIPTION = fp.read()
MAINTAINER = 'Dhaif BEKHA'
MAINTAINER_EMAIL = 'dhaif@dhaifbekha.com'
URL = 'https://conpagnon.github.io/conpagnon/'
LICENSE = 'new BSD'
DOWNLOAD_URL = 'https://github.com/ConPagnon/conpagnon/archive/v2.0.7.tar.gz'
VERSION = 'v2.0.7'


if __name__ == "__main__":
    setup(name=DISTNAME,
          maintainer=MAINTAINER,
          maintainer_email=MAINTAINER_EMAIL,
          description=DESCRIPTION,
          license=LICENSE,
          url=URL,
          version=VERSION,
          download_url=DOWNLOAD_URL,
          long_description=LONG_DESCRIPTION,
          zip_safe=False,
          classifiers=[
              'Intended Audience :: Science/Research',
              'Intended Audience :: Developers',
              'License :: OSI Approved',
              'Programming Language :: C',
              'Programming Language :: Python',
              'Topic :: Software Development',
              'Topic :: Scientific/Engineering',
              'Operating System :: POSIX',
              'Operating System :: Unix',
              'Operating System :: MacOS',
              'Programming Language :: Python :: 3.5',
              'Programming Language :: Python :: 3.6',
              'Programming Language :: Python :: 3.7',
          ],
          packages=find_packages(),
          install_requires=['numpy',
                            'joblib',
                            'matplotlib',
                            'nibabel',
                            'pandas',
                            'scipy',
                            'statsmodels',
                            'seaborn',
                            'webcolors',
                            'PyPDF2',
                            'scikit-learn',
                            'nilearn',
                            'psutil',
                            'networkx',
                            'patsy',
                            'xlrd',
                            'tabulate'],
          python_requires='>=3.5',
          )



# -*- coding: utf-8 -*-
"""
Created on Wed Feb 11 16:03:13 2015

@author: cp243490
"""
import numpy as np
import time
import tempfile
from mulm import MUOLS


print "Create matrix"
start_time = time.time()

n = 1000
px = 4
py_noize = int(2e5)
py_noize = int(1e6)
py_info = 100

## Build data
beta = np.array([1, 0, .5] + [0] * (px - 4) + [2]).reshape((px, 1))
X = np.hstack([np.random.randn(n, px-1), np.ones((n, 1))]) # X with intercept
f = tempfile.mktemp()
Y = np.random.randn(n, py_info + py_noize)
# Causal model: add X on the first py_info variable
Y[:, :py_info] += np.dot(X, beta)
print "Save matrix to disk"
np.save(f, Y)
del Y

contrasts = np.identity(X.shape[1])

## Read data Y as a memory map
##############################
Y_memmap = np.load(f + '.npy', mmap_mode='r')

# univariate analysis: fit by blocks of 2**27 elements
muols = MUOLS(Y=Y_memmap, X=X)
time3 = time.time()
muols.fit(block=True, max_elements=2 ** 27)
time4 = time.time()
tvals, pvals, dfs = muols.t_test(contrasts=contrasts,
                                 pval=True,
                                 two_tailed=True)
print  "Mmap by block(max_elements=2 ** 27): ", time4 - time3
del muols

# univariate analysis: fit in one go
muols = MUOLS(Y=Y_memmap, X=X)
time5 = time.time()
muols.fit(block=False)
time6 = time.time()
tvals, pvals, dfs = muols.t_test(contrasts=contrasts,
                                 pval=True,
                                 two_tailed=True)

print "Mmap no block: ", time6 - time5
del muols
del Y_memmap

## Load Y
##########
Y = np.load(f + '.npy')

# univariate analysis: fit by blocks of 2**27 elements
muols = MUOLS(Y=Y, X=X)
time9 = time.time()
muols.fit(block=True, max_elements=2 ** 27)
time10 = time.time()
tvals, pvals, dfs = muols.t_test(contrasts=contrasts,
                                 pval=True,
                                 two_tailed=True)
print "Mumpy by block(max_elements=2 ** 27): ",time10 - time9
del muols

# univariate analysis: fit in one go
muols = MUOLS(Y=Y, X=X)
time11 = time.time()
muols.fit(block=False)
time12 = time.time()
tvals, pvals, dfs = muols.t_test(contrasts=contrasts,
                                 pval=True,
                                 two_tailed=True)
print "Mumpy no block", time12 - time11
del muols
del Y


import numpy as np
import os
from nilearn import image
from nilearn import plotting
import pandas as pd
import webcolors
import nibabel as nb

# !/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Sep 11 15:07:18 2017

ComPagnon version 2.0

@author: db242421
"""

"""

Atlas class for easy manipulation of atlas.
"""


class Atlas:
    """A class Atlas for computing useful information when dealing with
    atlases.
    
    Methods
    -------
    
    fetch_atlas
        Return the complete path to the atlas.
        
    loadAtlas
        Load the atlas images and return a 4D numpy array.
        
    GetRegionNumbers
        Return the numbers of regions in the Atlas.
        
    GetLabels
        Return the list of the labels of the atlas.
        
    GetCenterOfMass
        Return tha array of coordinates of center of mass to
        each atlas regions.

    UserLabelsColors
        Generate an array of users defined colors to the 
        labels for display purpose.
    
    """
    
    def __init__(self, path, name):
        """Class constructor : an atlas file is defined by it's name and
        absolute path.
        
        Parameters
        ----------
        
        path : str
            The absolute path to the atlas file.
            
        name : str
            The name of the atlas file including it's extension.
        
        """
        self.path = path
        self.name = name
        
    def fetch_atlas(self):
        """Fetch the complete path to the atlas file.
                
        """
        return os.path.join(self.path, self.name)
        
    def load_atlas(self):
        """Load the atlas image and return the corresponding
        4D numpy array.
        
        """
        
        atlas_obj = image.load_img(os.path.join(self.path,self.name))
        atlas_data = atlas_obj.get_data()
        return atlas_data
    
    def get_region_numbers(self):
        
        """Return the number of regions of the atlas
        
        """
        atlas_data = self.load_atlas()
        region_numbers = atlas_data.shape[3]
        return region_numbers
        
    def GetLabels(self, labelsFile, colname='labels'):
        """Read the labels text file of the atlas
        
        Parameters
        ----------
        
        labelsFile : str
            The full path to the label file of the atlas. Supported
            extension are : .csv, .txt, .xlsx or .xls. By default,
            the header of the labels file is the column labels name 
            entitled 'labels'.
            
        colname : str, optional
            The columns name containing the labels. Default is labels.
            If no header, leave None.
            
        Returns
        -------
        
        output : list
            The list of the labels.
        
        """
        
        if labelsFile.lower().endswith(('.csv', '.txt')):
            labels = pd.read_csv(labelsFile)
        elif labelsFile.lower().endswith(('.xlsx', '.xls')):
            labels = pd.read_excel(labelsFile)
            
        if colname:
            labels = labels[colname]
            
        return labels
        
    def get_center_of_mass(self, asanarray=False):
        """Compute centers of mass of the different atlas regions.
        
        Parameters
        ----------
        asanarray : bool, optional
            If True, then the array are return if numpy.array of 
            shape (number of regions, 3).
            
        Returns
        -------
        output : list or numpy.array
            The coordinates of the centers of mass for each regions 
            of the atlas.

        """
        
        regions_center = [plotting.find_xyz_cut_coords(roi) for roi in image.iter_img(
            imgs=os.path.join(self.path, self.name))]
    
        if asanarray:
            regions_center = np.array(regions_center)
        
        return regions_center

    def user_labels_colors(self, networks, colors):
        """Generates user defined labels colors for each label of the atlas.
        
        Parameters 
        ----------
        
        networks : list
            The list containing the numbers of regions for each networks.
            
        colors : list
            The list of colors for each networks.
            
        Returns
        -------
        
        output : numpy.array, shape(number of regions, 3)
            The array containing the numerical values for the RGB space
            for the given colors entered in colors. For the normalized RGB space, 
            you just have divided the values of this array by 255.
            
        References
        ----------
        
        Please find all possible colors at
        [1] https://matplotlib.org/examples/color/named_colors.html
        
        """
        # Number of network detected
        n_networks = len(networks)
        # Number of colors, must match the number of network
        n_colors = len(colors)

        name_colors_to_rgb = [webcolors.name_to_rgb(colors[i]) for i in range(n_colors)]
        # convert string color name to rgb triplet
        rgb_labels_colors = np.array(name_colors_to_rgb)
        # Fill an array containing the colors for each regions, according to
        # network groups
        labels_colors = np.zeros([1,3])
        for i in range(n_networks):
            # temporary array for current networks labels
            tmp_labels_colors = np.zeros([networks[i], 3])
            # Fill the temporary array with the labels corresponding to the current network
            tmp_labels_colors[:] = rgb_labels_colors[i]
            # We append the current network labels colors array to the general array
            labels_colors = np.append(labels_colors, tmp_labels_colors,axis=0)
        # Delete the first line
        labels_colors = np.delete(labels_colors, 0, axis=0)
        return labels_colors


def fetch_atlas_functional_network(atlas_excel_file, sheetname, network_column_name):
    """Return a dictionary containing information for all functional networks.
       Information for all network is simply fetch in the excel file.

    Parameters
    ----------
    atlas_excel_file: str
        The full path to the excel file containing all the information on your atlas.
    sheetname: str
        The active sheet name in the atlas excel file.
    network_column_name: str
        The name of columns containing the label for all the functional networks.

    Returns
    -------
    output: dict
        A dictionnary with the networks name as keys, and the sub-dataframe and the number
        of roi for each networks as values.
    """
    df = pd.read_excel(atlas_excel_file, sheetname=sheetname)

    # Find the different network
    # List of networks
    networks_name_list = list(set(df[network_column_name]))

    # Create a dictionary with networks name as keys, and all the corresponding information columns
    # as values.
    networks_dictionary = dict.fromkeys(networks_name_list)
    # Group by functional network according to the labels in network_column_name
    df_group_by_networks = df.groupby(network_column_name)
    # Save the sub-dataframe corresponding to each network in the networks_dictionary.
    for network in networks_name_list:
        network_df = df_group_by_networks.get_group(name=network)
        networks_dictionary[network] = {'dataframe': network_df, 'number of rois': len(network_df.index)}

    return networks_dictionary


def generate_3d_img_network(reference_4datlas, atlas_information_xlsx_file, network_column_name,
                            sheetname, atlas4d_index_keys, atlas3d_label_key,
                            save_network_img_directory):
    """This function generate a 3D NifTi file for each defined functional network in a 4D atlas.

    """
    # Fetch network information for all networks
    networks_dictionnary = fetch_atlas_functional_network(
        atlas_excel_file=atlas_information_xlsx_file,
        sheetname=sheetname, network_column_name=network_column_name)

    # Load the atlas to get the shape and affine of images
    atlas4d = image.load_img(reference_4datlas)
    atlas4d_array = atlas4d.get_data()
    atlas_affine = atlas4d.affine
    atlas_shape = atlas4d.shape[:3]

    for network in networks_dictionnary.keys():
        # Get the number of rois
        n_roi_network = networks_dictionnary[network]['number of rois']
        # Initialize a 4D array containing the roi of the current network in 4D dim
        network4d = np.zeros(atlas_shape + (n_roi_network,))
        # Fetch the number of current network rois in the 4D atlas file
        network_roi_number = networks_dictionnary[network]['dataframe'][atlas4d_index_keys]
        # Find the corresponding 3D label of the network, because network_roi_number is a dataframe
        # we use the iloc method to find the corresponding value for the network_roi_number INDEX in the
        # right order.
        network_roi_index = network_roi_number.index
        network_3d_labels = list(networks_dictionnary[network]['dataframe'][atlas3d_label_key].loc[network_roi_index])
        # Fill the network 4D array with the corresponding 3D labels of each roi
        for r in range(n_roi_network):
            # Fill the 4d network array with the corresponding rois of the current network
            network4d[:, :, :, r] = atlas4d_array[:, :, :, list(network_roi_number)[r]]
            # Put the right 3D label for each roi in the current network
            x, y, z = np.where(network4d[:, :, :, r] != 0)
            network4d[x, y, z, r] = network_3d_labels[r]
            # Sum along the 4d dim to get one 3D image for the current network
            network_3d_array = np.sum(network4d, axis=3)
            # Save the network 3D image in NiFti format
            nb.save(img=nb.Nifti1Image(dataobj=network_3d_array, affine=atlas_affine),
                    filename=os.path.join(save_network_img_directory, '3d_labels_' + network + '.nii'))


def closest_colour(requested_colour):
    min_colours = {}
    for key, name in webcolors.css3_hex_to_names.items():
        r_c, g_c, b_c = webcolors.hex_to_rgb(key)
        rd = (r_c - requested_colour[0]) ** 2
        gd = (g_c - requested_colour[1]) ** 2
        bd = (b_c - requested_colour[2]) ** 2
        min_colours[(rd + gd + bd)] = name
    return min_colours[min(min_colours.keys())]


def get_colour_name(requested_colour):
    try:
        closest_name = actual_name = webcolors.rgb_to_name(requested_colour)
    except ValueError:
        closest_name = closest_colour(requested_colour)
        actual_name = None
    return actual_name, closest_name


def fetch_atlas(atlas_folder, atlas_name,
                colors_labels='auto',
                network_regions_number='auto',
                labels='auto',
                normalize_colors = False):
    """Return important information from an atlas file.

    Parameters
    ----------
        atlas_folder: str
            The full path to the directory containing the atlas
        atlas_name: str
            The filename of the atlas file.
        colors_labels: str, or list. Optional
            If set to 'auto', the labels of the
            ROI name will get a random colors. Else,
            if a list of colors is provided, ROIs belonging
            to a network will get the desired colors. The colors
            should be in the same order of the network in the atlas
            file. The length of the list should match the number of
            network in the atlas.
        network_regions_number: list, Optional.
            If set to 'auto', random colors will be chosen.
            If a list of the number of regions in each network
            is provided, the corresponding color, in the color list
            will be applied to the corresponding number in the list.
        labels: list, optional
            The list of the ROI labels. If not provided,
            the ROI name is simply it's position in the atlas
            file.
        normalize_colors: bool, optional
            If True, all triplets in the RGB space are
            divided by the maximum 255.

    Returns
    -------
    output 1: numpy.array
        The coordinates of the center of mass of
        each ROI in the atlas. An array of shape (n_rois, 3).
    output 2: list
        The name of each ROIs in the atlas. A list
        of length (n_rois, ).
    output 3: numpy.array
        The array containing the colors in the
        RGB space of each ROIs. An Array of shape
        (n_rois, 3).
    output_4: int
        The number of ROIs in the atlas.

    """

    atlas_ = Atlas(path=atlas_folder,
                   name=atlas_name)

    # Fetch nodes coordinates
    atlas_nodes = atlas_.get_center_of_mass()
    # Fetch number of nodes in the atlas
    n_nodes = atlas_.get_region_numbers()
    # Atlas path
    if labels == 'auto':
        labels_regions = np.arange(n_nodes)
    else:
        labels_regions = atlas_.GetLabels(labels)
    # if the user give a list containing the number of regions in each networks,
    # but not the colors
    if (colors_labels == 'auto') & (type(network_regions_number) == list):
        n_network = len(network_regions_number)
        # Generate n_network random colors
        random_colors = np.random.randint(0, 256, size=(n_network, 3))
        # Convert list of triplet to name with webcolors
        regions_colors_name = [get_colour_name(random_colors[i])[1] for i in range(n_network)]
        # Generate the colors for each labels according to the network they belong
        labels_colors = atlas_.user_labels_colors(networks=network_regions_number,
                                                  colors=regions_colors_name)
        if normalize_colors:
            labels_colors = (1/255)*labels_colors
    # if the user doesn't give any colors labels nor networks region number
    # we create random color for each regions labels
    elif (colors_labels == 'auto') & (network_regions_number == 'auto'):
        # Generate n_network random colors
        random_colors = np.random.randint(0, 256, size=(n_nodes, 3))
        # Convert list of triplet to name with webcolors
        regions_colors_name = [get_colour_name(random_colors[i])[1] for i in range(n_nodes)]
        # Generate the colors for each labels according to the network they belong
        labels_colors = regions_colors_name
        if normalize_colors:
            labels_colors = (1/255)*np.array(labels_colors)
    else:
        labels_colors = atlas_.user_labels_colors(networks=network_regions_number,
                                                  colors=colors_labels)
        if normalize_colors:
            labels_colors = (1/255)*labels_colors

    return atlas_nodes, labels_regions, labels_colors, n_nodes





"""
This module is designed to study the link between
functional connectivity and behaviour. The algorithm
named connectome predictive modelling is adapted from
[1].

.. [1] Using connectome-based predictive modeling to
predict individual behavior from brain connectivity, Shen et al.

author: Dhaif BEKHA.
"""
import numpy as np
from conpagnon.pylearn_mulm import mulm
from scipy import stats
from scipy.stats import t
import statsmodels.api as sm
from conpagnon.connectivity_statistics.parametric_tests import partial_corr
from sklearn.model_selection import LeaveOneOut

"""
This module contain useful function for the connectome predictive modelling algorithm.


Author: Dhaif BEKHA.
"""


def predictors_selection_linear_model(training_connectivity_matrices,
                                      training_confound_variable_matrix,
                                      training_set_behavioral_score):
    """Relate each edges of subjects connectivity matrices in the training set
    with a behavioral scores using a linear model


    """
    # Fit linear model at each edge
    n_edge = training_connectivity_matrices.shape[1]
    t_stats = np.zeros(n_edge)
    p_vals = np.zeros(n_edge)
    df_ = np.zeros(n_edge)
    for i in range(n_edge):
        X_train_edge = np.c_[np.ones(training_connectivity_matrices.shape[0]),
                             training_connectivity_matrices[:, i],
                             np.array(training_confound_variable_matrix)]
        # Fit a linear model
        training_set_model = mulm.MUOLS(training_set_behavioral_score, X_train_edge)
        contrasts = np.identity(X_train_edge.shape[1])
        t_value, p_value, df = training_set_model.fit().t_test(contrasts, pval=True, two_tailed=True)
        df_[i] = df[1]

        # Append t value for the principal predictor of interest, i.e the connectivity coefficient
        t_stats[i] = t_value[1, :]
        p_vals[i] = p_value[1, :]

    # For each edges convert the t statistic of the linear model in correlation
    # coefficient value
    R_mat = np.sign(t_stats) * np.sqrt(t_stats**2 / (df_[0] + t_stats**2))
    P_mat = 2*t.sf(np.abs(t_stats), df_[0])

    return R_mat, P_mat


def predictors_selection_correlation(training_connectivity_matrices, 
                                     training_set_behavioral_scores):
    R_mat = np.zeros(training_connectivity_matrices.shape[1])
    P_mat = np.zeros(training_connectivity_matrices.shape[1])

    for i in range(training_connectivity_matrices.shape[1]):
        # Simple correlation between each edges and behavior
        R_mat[i], P_mat[i] = stats.pearsonr(x=training_set_behavioral_scores[:, 0],
                                            y=training_connectivity_matrices[:, i])

    return R_mat, P_mat


def predictor_selection_pcorrelation(training_connectivity_matrices,
                                     training_set_behavioral_scores,
                                     training_set_confounding_variables):
    # Matrix which will contain the correlation of each edge to behavior, and the corresponding
    # p values
    R_mat = np.zeros(training_connectivity_matrices.shape[1])

    # Construct temporary array to contain the connectivity, behavior and
    # other variable to regress
    for i in range(training_connectivity_matrices.shape[1]):
        R_ = partial_corr(np.c_[training_connectivity_matrices[:, i],
                                training_set_behavioral_scores[:, 0],
                                training_set_confounding_variables])
        R_mat[i] = R_[0, 1]

    df = training_connectivity_matrices.shape[0] - 2 - training_set_confounding_variables.shape[1]
    # Compute p values manually: convert R values in t statistic
    t_mat = (np.sqrt(df) * np.abs(R_mat)) / (np.sqrt(np.ones(R_mat.shape[0]) - R_mat ** 2))
    # Compute two tailed p value
    P_mat = t.sf(np.abs(t_mat), df) * 2

    return R_mat, P_mat


def compute_summary_subjects_summary_values(training_connectivity_matrices,
                                            significance_selection_threshold,
                                            R_mat, P_mat):
    # Positive and Negative correlation indices, under the selection threshold
    negative_edges_indices = np.nonzero((R_mat < 0) & (P_mat < significance_selection_threshold))
    positives_edges_indices = np.nonzero((R_mat > 0) & (P_mat < significance_selection_threshold))

    negative_edges_mask = np.zeros(training_connectivity_matrices.shape[1])
    positive_edges_mask = np.zeros(training_connectivity_matrices.shape[1])

    # Fill the corresponding indices with 1 if indices exist, zero elsewhere
    negative_edges_mask[negative_edges_indices] = 1
    positive_edges_mask[positives_edges_indices] = 1

    # Get the sum off all edges in the mask
    negative_edges_summary_values = np.zeros(training_connectivity_matrices.shape[0])
    positive_edges_summary_values = np.zeros(training_connectivity_matrices.shape[0])

    for i in range(training_connectivity_matrices.shape[0]):
        negative_edges_summary_values[i] = np.sum(np.multiply(negative_edges_mask,
                                                              training_connectivity_matrices[i, ...]))
        positive_edges_summary_values[i] = np.sum(np.multiply(positive_edges_mask,
                                                              training_connectivity_matrices[i, ...]))

    return negative_edges_mask, positive_edges_mask, negative_edges_summary_values, \
           positive_edges_summary_values


def fit_model_on_training_set(negative_edges_summary_values, positive_edges_summary_values,
                              training_set_behavioral_score, add_predictive_variables=None):
    """Fit a linear model on the training set for positive and negative model, with behavioral score
    as variable.

    Parameters
    ----------
    negative_edges_summary_values: numpy.array, shape(n_subjects_in_training_set, )
        The sum of predictors for each subject in the training set
        having a negative correlation with the behavioral scores.
    positive_edges_summary_values: numpy.array, shape(n_subjects_in_training_set, )
        The sum of predictors for each subject in the training set
        having a positive correlation with the behavioral scores.
    training_set_behavioral_score: numpy.array, shape (n_subjects_in_training_set, 1)
        The behavioral scores of the training set.
    add_predictive_variables: pandas.DataFrame, optional
        If not None, a pandas DataFrame of shape (n_subjects_in_training_set, n_variables) should
        be given. The variables wil be used in the predictive model.

    """
    # Fit a linear model on the training set, for negative and positive edges summary values
    design_matrix_negative_edges = sm.add_constant(negative_edges_summary_values)

    design_matrix_positive_edges = sm.add_constant(positive_edges_summary_values)

    # if the user add additional to the predictive model we concatenate
    # the additional variables matrix to the existing one containing the intercept
    # and negatives/positive edges summary values.
    if add_predictive_variables is not None:

        design_matrix_negative_edges = np.c_[design_matrix_negative_edges, add_predictive_variables]
        design_matrix_positive_edges = np.c_[design_matrix_positive_edges, add_predictive_variables]
    else:
        design_matrix_negative_edges = design_matrix_negative_edges
        design_matrix_positive_edges = design_matrix_positive_edges

    # Fit positive edges model
    positive_edges_model = sm.OLS(training_set_behavioral_score, design_matrix_positive_edges)
    positive_edge_model_fit = positive_edges_model.fit()

    # Fit negative edges model
    negative_edges_model = sm.OLS(training_set_behavioral_score, design_matrix_negative_edges)
    negative_edge_model_fit = negative_edges_model.fit()

    return positive_edge_model_fit, negative_edge_model_fit


def predict_behavior(vectorized_connectivity_matrices, behavioral_scores,
                     selection_predictor_method='correlation',
                     significance_selection_threshold=0.01,
                     confounding_variables_matrix=None,
                     add_predictive_variables=None,
                     verbose=0):
    """Predict behavior from matrix connectivity with a simple linear model.

    Parameters
    ----------
    vectorized_connectivity_matrices: numpy.array, shape (n_sample, n_features)
        The subjects connectivity matrices in a vectorized form, i.e, you must
        give a array of shape (n_subjects, 0.5*n_features*(n_features-1)).
    behavioral_scores: pandas.DataFrame or numpy.array of shape(n_subjects, 1)
        The behavioral scores in a pandas DataFrame or numpy array form. Off course
        the behavioral scores should ordered in the same order of subjects matrices.
    selection_predictor_method: str, optional
        The selection method of predictors. When relate behavior and functional connectivity
        multiple choices are possible: simple correlation, partial correlation or linear_model.
        Default is correlation. Note that partial correlation or linear model should give
        the same results.
    significance_selection_threshold: float, optional
        The threshold level for the p-value resulting of the selection of predictors
        step. Default is 0.01.
    confounding_variables_matrix: pandas.DataFrame of shape (n_subjects, n_variables), optional
        A dataframe  of shape (n_subjects, n_variables) containing the confounding/controlling
        variable which might be used in the selection predictors step when selection method is
        partial correlation/linear regression. Defaults is None.
    add_predictive_variables: pandas.DataFrame shape (n_subjects_in_training_set, n_variables) or None, optional
        If not None, additional variables will be fitted in the predictive model, besides negative
        and positive summary features.
    verbose: int, optional
        If verbose equal to 0 nothing is printed.

    Returns
    -------
    output 1: float
        The correlation coefficient for the positive features model.
    output 2: float
        The correlation coefficient for the negative features model.




    """
    # Initialize leave one out object
    leave_one_out_generator = LeaveOneOut()

    # Initialization of behavior prediction vector
    behavior_prediction_positive_edges = np.zeros(vectorized_connectivity_matrices.shape[0])
    behavior_prediction_negative_edges = np.zeros(vectorized_connectivity_matrices.shape[0])

    all_selected_positive_features = []
    all_selected_negative_features = []

    for train_index, test_index in leave_one_out_generator.split(vectorized_connectivity_matrices):
        if verbose:
            print('Train on subjects # {}'.format(train_index))
            print('Test on subject # {}'.format(test_index))
        # For each iteration, split the patients matrices array in train and
        # test set using leave one out cross validation
        patients_train_set, leave_one_out_patients = \
            vectorized_connectivity_matrices[train_index], vectorized_connectivity_matrices[test_index]

        # Training set behavioral scores
        training_set_behavioral_score_ = np.zeros((patients_train_set.shape[0], 1))
        training_set_behavioral_score_[:, 0] = behavioral_scores[train_index]

        # The confounding variables, stored in an array for the training set
        if confounding_variables_matrix is not None:
            training_confound_variable_matrix = confounding_variables_matrix.iloc[train_index]
        else:
            training_confound_variable_matrix = None

        # The additional variable for the predictive model
        if add_predictive_variables is not None:
            training_additional_predictive_variables = add_predictive_variables.iloc[train_index]
            test_subject_additional_predictive_variables = add_predictive_variables.iloc[test_index]
        else:
            training_additional_predictive_variables = None
            test_subject_additional_predictive_variables = None

        if selection_predictor_method == 'linear_model':

            # Correlation of each edge to the behavioral score for training set
            R_mat, P_mat = predictors_selection_linear_model(
                training_connectivity_matrices=patients_train_set,
                training_confound_variable_matrix=training_confound_variable_matrix,
                training_set_behavioral_score=training_set_behavioral_score_)

            # Compute summary values for both positive and negative edges model
            negative_edges_mask, positive_edges_mask, negative_edges_summary_values, positive_edges_summary_values = \
                compute_summary_subjects_summary_values(
                    training_connectivity_matrices=patients_train_set,
                    significance_selection_threshold=significance_selection_threshold,
                    R_mat=R_mat, P_mat=P_mat)

            all_selected_negative_features.append(negative_edges_mask)
            all_selected_positive_features.append(positive_edges_mask)

        elif selection_predictor_method == 'correlation':

            R_mat, P_mat = \
                predictors_selection_correlation(training_connectivity_matrices=patients_train_set,
                                                 training_set_behavioral_scores=training_set_behavioral_score_)

            negative_edges_mask, positive_edges_mask, negative_edges_summary_values, positive_edges_summary_values = \
                compute_summary_subjects_summary_values(
                    training_connectivity_matrices=patients_train_set,
                    significance_selection_threshold=significance_selection_threshold,
                    R_mat=R_mat, P_mat=P_mat)

            all_selected_negative_features.append(negative_edges_mask)
            all_selected_positive_features.append(positive_edges_mask)

        elif selection_predictor_method == 'partial correlation':
            if confounding_variables_matrix is None:
                raise ValueError('Confound variables is empty !')
            else:
                R_mat, P_mat = predictor_selection_pcorrelation(
                    training_connectivity_matrices=patients_train_set,
                    training_set_behavioral_scores=training_set_behavioral_score_,
                    training_set_confounding_variables=training_confound_variable_matrix)

                negative_edges_mask, positive_edges_mask, negative_edges_summary_values, \
                    positive_edges_summary_values = \
                    compute_summary_subjects_summary_values(
                        training_connectivity_matrices=patients_train_set,
                        significance_selection_threshold=significance_selection_threshold,
                        R_mat=R_mat, P_mat=P_mat)

                all_selected_negative_features.append(negative_edges_mask)
                all_selected_positive_features.append(positive_edges_mask)
        else:
            raise ValueError('Selection method not understood')

        # Fit a linear model on the training set
        positive_edge_model_fit, negative_edge_model_fit = fit_model_on_training_set(
            negative_edges_summary_values=negative_edges_summary_values,
            positive_edges_summary_values=positive_edges_summary_values,
            training_set_behavioral_score=training_set_behavioral_score_,
            add_predictive_variables=training_additional_predictive_variables)

        # Compute summary statistic for the left out subject
        test_subject_positive_edges_summary = np.sum(np.multiply(leave_one_out_patients[0, :],
                                                                 positive_edges_mask))
        test_subject_negative_edges_summary = np.sum(np.multiply(leave_one_out_patients[0, :],
                                                                 negative_edges_mask))

        # Add additional predictive variables for the test subject
        if add_predictive_variables is not None:
            test_subject_variable_positive_edges = np.c_[np.ones(1), test_subject_positive_edges_summary,
                                                         test_subject_additional_predictive_variables]
            test_subject_variable_negative_edges = np.c_[np.ones(1), test_subject_negative_edges_summary,
                                                         test_subject_additional_predictive_variables]

        else:
            test_subject_variable_positive_edges = np.c_[np.ones(1), test_subject_positive_edges_summary]
            test_subject_variable_negative_edges = np.c_[np.ones(1), test_subject_negative_edges_summary]

        # Fit the model of on the left out subject
        behavior_prediction_negative_edges[test_index] = \
            negative_edge_model_fit.predict(test_subject_variable_negative_edges)

        behavior_prediction_positive_edges[test_index] = \
            positive_edge_model_fit.predict(test_subject_variable_positive_edges)

    # Compare prediction and true behavioral score
    R_predict_negative_model, _ = \
        stats.pearsonr(x=behavior_prediction_negative_edges,
                       y=np.array(behavioral_scores))

    R_predict_positive_model, _ = \
        stats.pearsonr(x=np.array(behavioral_scores),
                       y=behavior_prediction_positive_edges)

    return (R_predict_positive_model, R_predict_negative_model, all_selected_positive_features,
            all_selected_negative_features)



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Thu Sep 21 13:34:12 2017

@author: db242421


Utilitary modules.

"""


import os
import pickle
from PyPDF2 import PdfFileWriter, PdfFileReader
import shutil


def remove_empty_directories(root_directory):

    """Erase all empty directory at a root location.

    Parameters
    ----------
    root_directory : str
        The full path to a directory.

    """
    sub_dir_list = os.listdir(root_directory)

    for sub_dir in sub_dir_list:
        if not os.listdir(os.path.join(root_directory, sub_dir)):
            os.rmdir(os.path.join(root_directory, sub_dir))


def check_directories_existence(root_directory, directories_list):
    """Check if all directories supplied exist, raises an exception if not.

    Parameters
    ----------
    root_directory : str
        The full path to a root directory, containing multiple folders.

    directories_list : list
        List of directories you want to check for, in root_directory.

    Raises
    ------
    Exception
        Raise an exception if one of the directories supplied
        do not exist.

    """

    # la liste des dossiers a la racine root_directory
    sub_dir_list = os.listdir(root_directory)

    for d in directories_list:
        if d not in sub_dir_list:
            raise Exception('le dossier {} n\'existe pas ! Les dossiers '
                            'disponibles dans {} sont {}'.format(d, root_directory, sub_dir_list))


def save_object(object_to_save, saving_directory, filename):
    """Save a python object in pickle format.

    Parameters
    ----------
    object_to_save : a Python object structure
        A python that can be pickled : list, dict, integer,
        tuples, etc...

    saving_directory : str
        Full path to a saving directory.

    filename : str
        Filename of the saved object, including the extension .pkl.

    See Also
    --------
    load_object :
        This function load a python object save
        by this function.

    Notes
    -----
    See the pickle module for python object to see
    a exhausting list of what object can be saved.

    """
    full_path_to_file = os.path.join(saving_directory, filename)
    with open(full_path_to_file, 'wb+') as output:
        pickle.dump(obj=object_to_save, file=output, protocol=pickle.HIGHEST_PROTOCOL )


def load_object(full_path_to_object):

    """Load a python object saved by a pickler operator.

    Parameters
    ----------
    full_path_to_object : str
        The full path to the saved python object, including
        the .pkl extension.

    Returns
    -------
    output : python object
        The loaded python object.

    See Also
    --------
    save_object :
        This function save a python object
        using the pickle module.

    """
    with open(full_path_to_object, 'rb') as pickleObj:

        object_to_load = pickle.load(pickleObj)
    return object_to_load


# Creating a routine that appends files to the output file
def append_pdf(input, output):
    """Append a input pdf file to an output empty pdf file.

    """
    [output.addPage(input.getPage(page_num)) for page_num in range(input.numPages)]


def merge_pdfs(pdfs, output_filename):
    """Merge together a list of pdf files.

    """

    # Creating an object where pdf pages are appended to
    output = PdfFileWriter()

    # Appending two pdf-pages from two different files
    for pdf in pdfs:
        # read the pdf file
        pdf_ = PdfFileReader(pdf)
        append_pdf(pdf_,output)

    # Writing all the collected pages to a file
    output.write(open(output_filename, "wb"))


def create_directory(directory, erase_previous=False):
    """Create a directory, erase it and create it
    if it exist.
    """

    # If directory doesn't exist we create it
    if not os.path.exists(directory):
        os.makedirs(directory)
    else:
        # It it exist, and erase_previous is True we erase it
        if erase_previous:
            shutil.rmtree(directory)
            os.makedirs(directory)

    return directory



# Configuration file for the Sphinx documentation builder.
#
# This file only contains a selection of the most common options. For a full
# list see the documentation:
# http://www.sphinx-doc.org/en/master/config
# Created by Dhaif BEKHA (dhaif@dhaifbekha.com)

# -- Path setup --------------------------------------------------------------

# If extensions (or modules to document with autodoc) are in another directory,
# add these directories to sys.path here. If the directory is relative to the
# documentation root, use os.path.abspath to make it absolute, like shown here.
#
import os
import sys
sys.path.insert(0, os.path.abspath('../../'))
import sphinx_material

# -- General configuration -----------------------------------------------------

# If your documentation needs a minimal Sphinx version, state it here.

# Add any Sphinx extension module names here, as strings. They can be
# extensions coming with Sphinx (named 'sphinx.ext.*') or your custom ones.
extensions = ['sphinx.ext.autodoc',
              # numpydoc or sphinx.ext.napoleon, but not both
              'sphinx.ext.napoleon',
             # 'numpydoc',
              'sphinx.ext.doctest',
              'sphinx.ext.extlinks',
              'sphinx.ext.intersphinx',
              'sphinx.ext.todo',
              'sphinx.ext.autosectionlabel',
              # One of mathjax or imgmath
              'nbsphinx',
              'sphinx.ext.mathjax',
              'sphinx.ext.viewcode',
              # 'sphinx.ext.autosummary',
              'sphinx.ext.inheritance_diagram',
              'matplotlib.sphinxext.plot_directive',
              'IPython.sphinxext.ipython_console_highlighting',
              'IPython.sphinxext.ipython_directive',
              'sphinx_gallery.gen_gallery'
              ]

try:
    import sphinxcontrib.spelling  # noqa: F401
except ImportError as err:  # noqa: F841
    pass
else:
    extensions.append('sphinxcontrib.spelling')

# nbsphinx options
nbsphinx_allow_errors = True
# sphinxcontrib-spelling options
spelling_word_list_filename = ['spelling_wordlist.txt', 'names_wordlist.txt']
spelling_ignore_pypi_package_names = True

# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.


# Add any paths that contain custom static files (such as style sheets) here,
# relative to this directory. They are copied after the builtin static files,
# so a file named "default.css" will overwrite the builtin "default.css".
# html_static_path = ['_static']

# The suffix of source filenames.
source_suffix = '.rst'

# The master toctree document.
master_doc = 'index'

# General information about the project.
project = u'ConPagnon 2.0'
version = u'2.0'
copyright = u'2016 - 2020. Created by Dhaif BEKHA.'
autosummary_generate = True
autoclass_content = 'class'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
exclude_patterns = ['_build', '**.ipynb_checkpoints', '*/autosummary/*.rst',
                    'Thumbs.db', '.DS_Store']

# The name of the Pygments (syntax highlighting) style to use.
pygments_style = 'default'

# Options for HTML output

# The theme to use for HTML and HTML Help pages.  See the documentation for
# a list of builtin themes.
extensions.append('sphinx_material')
html_theme_path = sphinx_material.html_theme_path()
html_context = sphinx_material.get_html_context()
html_theme = 'sphinx_material'
html_title = project
html_short_title = project

# material theme options (see theme.conf for more information)
base_url = 'https://conpagnon.github.io/conpagnon/'
html_theme_options = {
    'base_url': base_url,
    'repo_url': 'https://github.com/ConPagnon/conpagnon',
    'repo_name': 'ConPagnon',
    'globaltoc_depth': 3,
    'globaltoc_collapse': True,
    'globaltoc_includehidden': True,
    'color_primary': 'red',
    'color_accent': 'blue',
    'nav_title': 'ConPagnon {0}'.format(version),
    'master_doc': False,
    'nav_links': [],
    'heroes': {'index': 'Easy resting state analysis in Python ',
               'examples/index': 'A series of examples to get you started with ConPagnon'},
    "version_dropdown": True,
    "version_json": "_static/versions.json",
}

language = 'en'
html_last_updated_fmt = ''

# ConPagnon Logo
html_logo = 'images/logo/logo_conpagnon_small_header.png'

# Custom sidebar templates, maps document names to template names.
html_sidebars = {
    "**": ["logo-text.html", "globaltoc.html", "localtoc.html", "searchbox.html"]
}

# If false, no module index is generated.
html_domain_indices = True

# Create xrefs
numpydoc_use_autodoc_signature = True
numpydoc_xref_param_type = True
numpydoc_class_members_toctree = False

# Configuration of Sphinx Gallery paths: those
# paths are relative to the sphinx folder.
sphinx_gallery_conf = {
     'examples_dirs': '../../examples',   # path to your example scripts
     'gallery_dirs': './examples',  # path to where to save gallery generated output
}
